package VR.my_util

import VR.OptionMap

/**
  * Created by YY on 2016/1/24.
  */
object Runner {
  val usage = """
Usage: Runner --alg xx --eta xx --batch xx --dataname xx --datafolder xx
       [--split] [--cluster xx] [--sigma xx] [--lambda xx] [--crossvalid xx] [--testdataname xx]

 --alg: SVRG/SAGA/SAG/UNIVR/SDCA
 --eta: recommend choosing from [1e-5,1]
 --batch: recommend 30, could be more
 --dataname: say, a9a/MNIST/covtype/news20.binary
 --datafolder: say, data
 [--split]: Without parameters. Means split the train data random to get both test data and train data.
    It can not use with --cluster
 [--cluster xx]: say, a9a-1-2800.dat. Means read cluster information from "dataname\\a9a-1-2800.dat"
    It can't be used with split/crossvalid, because cluster information is damaged.
 [--sigma xx]: sigma parameter for Lasso.
 [--lambda xx]: lambda parameter for ridge.
 [--crossvalid xx]: cross validation for some parameter. Currently could be either "sigma" or "lambda".
    One and only one of --sigma/--lambda/--crossvalid should be used.
 [--testdataname xx]: say, a9a-test. It means reading data from "datafolder\\testdataname.bat"
              """
  def main(args: Array[String]) {
    for (arg<-args)
      print(arg+" ")
    println
    if (args.length == 0) println(usage)

    def remove_element( list: List[String], item:String) = list diff List(item)
    def nextOption(map : OptionMap, list: List[String], requires: List[String]) : (OptionMap, List[String]) = {
      list match {
        case Nil => (map, requires)
        case "--alg" :: value :: tail =>
          if (value.toString.toUpperCase=="SDCA")
            nextOption(map ++ Map('algorithm -> value.toString.toUpperCase)++Map('eta->0.0), tail, remove_element(remove_element (requires,"alg"),"eta"))
          else
            nextOption(map ++ Map('algorithm -> value.toString.toUpperCase), tail, remove_element (requires,"alg"))
        case "--eta" :: value :: tail=>
          nextOption(map ++ Map('eta -> value.toFloat), tail, remove_element(requires,"eta"))
        case "--sigma" :: value :: tail=>
          nextOption(map ++ Map('sigma -> value), tail, remove_element(requires,"sigma or lambda or crossvalid"))
        case "--lambda" :: value :: tail=>
          nextOption(map ++ Map('lambda -> value), tail, remove_element(requires,"sigma or lambda or crossvalid"))
        case "--batch" :: value :: tail=>
          nextOption(map ++ Map('batch -> value.toInt), tail, remove_element(requires,"batch"))
        case "--crossvalid" :: value :: tail=>
          nextOption(map ++ Map('cross -> value.toString.toLowerCase), tail, remove_element(requires,"sigma or lambda or crossvalid"))
        case "--dataname" :: value :: tail=>
          nextOption(map ++ Map('dataname -> value.toString), tail, remove_element(requires,"dataname"))
        case "--datafolder" :: value :: tail=>
          if (value.toString.last=='\\')
            nextOption(map ++ Map('datafolder -> value.toString), tail, remove_element(requires,"datafolder"))
          else
            nextOption(map ++ Map('datafolder -> value.toString.concat("\\")), tail, remove_element(requires,"datafolder"))
        case "--cluster" :: value :: tail =>
          nextOption(map ++ Map('cluster -> value.toString.toUpperCase), tail, requires)
        case "--split" :: tail =>
          nextOption(map ++ Map('split -> ""), tail, requires)
        case "--logistic" :: tail =>
          nextOption(map ++ Map('logistic -> ""), tail, requires)
        case "--reglambda" :: value :: tail=>
          nextOption(map ++ Map('reglambda -> value), tail, requires)
        case "--testdataname" :: value :: tail =>
          nextOption(map ++ Map('testdataname -> value.toString), tail, requires)
        case "--PCA0":: n :: d:: tail =>
          nextOption(map ++ Map('PCA -> 0)++ Map('n->n)++Map('d->d), tail, requires)
        case "--PCA1":: n:: d :: delta:: eta:: batch:: tail =>
          nextOption(map ++ Map('PCA -> 1)++ Map('n->n)++Map('d->d)++ Map('delta->delta)++ Map('eta -> eta) ++ Map('batch->batch), tail, requires)
        case "--PCA2":: n:: d :: delta:: eta:: batch:: tail =>
          nextOption(map ++ Map('PCA -> 2)++ Map('n->n)++Map('d->d)++ Map('delta->delta)++ Map('eta -> eta) ++ Map('batch->batch), tail, requires)
        case "--get_ans" ::tail=>
          nextOption(map ++ Map('get_ans-> ""), tail, requires)
        case "--recompute" ::tail=>
          nextOption(map ++ Map('recompute-> ""), tail, requires)
        case option :: tail => println("Unknown option "+option)
          (map, requires)
      }
    }
    val required_args=List("alg", "sigma or lambda or crossvalid", "eta", "dataname", "datafolder", "batch")
    val (options,remain_args) = nextOption(Map(),args.toList, required_args)
    if (!options.contains('PCA))
    if (remain_args.nonEmpty
      || (options.contains('sigma) && options.contains('lambda))
      || (options.contains('cross) && (!List("lambda","sigma").contains(options('cross))))
      || (options.contains('split) && options.contains('testdataname))
      || (options.contains('cluster) && options.contains('cross))
      || (options.contains('cluster) && options.contains('split))
      || (options.contains('cross) && options.contains('split))
      || (options.contains('testdataname) && options.contains('cross))
      || (options('algorithm)=="SDCA" && (!options.contains('lambda)) && (!options.contains('reglambda)))
    )
      {
        for (arg<-remain_args)
          println("Error: "+arg+" is required.")
        if (options.contains('sigma) && options.contains('lambda))
          println("Error: Elastic Nets not supported.")
        if (options.contains('cross) && (!List("lambda","sigma").contains(options('cross))))
          println("Error: cross validation parameter not known.")
        if (options.contains('cluster) && options.contains('cross))
          println("Error: cross validation does not support using clusters. ")
        if (options.contains('cluster) && options.contains('split))
          println("Error: split does not support using clusters. ")
        if (options.contains('cross) && options.contains('split))
          println("Error: split does not support cross validation. ")
        if (options.contains('testdataname) && options.contains('cross))
          println("Error: cross validation does not support test data. ")
        if (options.contains('split) && options.contains('testdataname))
          println("Error: I'm confused. Split or read from extra test data file?")
         if (options('algorithm)=="SDCA" && (!options.contains('lambda)) && (!options.contains('reglambda)))
          println("Error: SDCA needs strong convexity. ")
        return
      }

    if (options.contains('PCA))
      PCA_worker.do_pca_work(options)
    else
      ERM_worker.do_work(options)
  }
}
